# Importation
from parametres import *
from tuiles import TuileAnimée
import pygame as pg
from random import randint, choice

class Lucioles:
    '''
    Classe qui affiche des points lumineux sur l'écran de manière aléatoire, points lumineux qui se déplacent eux aussi aléatoirement.
    '''
    def __init__(self,largeur_niveau,hauteur_niveau,nb_lucioles,groupe):

        chemin = "../Visuels/décorations/lucioles/"

        # Images identiques mais qui commencent à des endroits différents dans l'animation pour éviter que les lucioles brillent toutes en même temps
        liste_lucioles = ["luciole1","luciole2","luciole3"]

        # Définit des abscisses minimale et maximale d'apparition
        self.x_min = -largeur_ecran
        self.x_max = largeur_niveau + largeur_ecran

        self.sprites_mouvement = pg.sprite.Group() # Groupe permettant de réaliser la méthode mouvement()
        self.hauteur_niveau = hauteur_niveau

        for luciole in range(nb_lucioles):
            # Création de lucioles au nombre définit lors de l'instance de la classe
            x = randint(self.x_min,self.x_max)
            y = randint(0,self.hauteur_niveau)
            luciole = chemin + choice(liste_lucioles) # Complétion du chemin vers un dossier d'images pris au hasard
            sprite = TuileAnimée(0,(x,y),luciole)

            # Ajout aux deux groupes
            groupe.add(sprite)
            self.sprites_mouvement.add(sprite)

    def mouvement(self):
        '''
        Parcourt l'ensemble des sprites de lucioles et téléporte des sprites au hasard dans le niveau 
        si la valeur de temps vaut la valeur arbitraire de 6 et que l'animation du sprite est terminée.
        '''
        for sprite in self.sprites_mouvement.sprites():
            temps = randint(0,35)
            if temps == 6 and sprite.index_images > len(sprite.images):
                x = randint(self.x_min,self.x_max)
                y = randint(0,self.hauteur_niveau)
                sprite.rect.topleft = (x,y)


class Feuilles:
    def __init__(self,largeur_niveau,hauteur_niveau,nb_feuilles,groupe):

        # Variables semblables à la classe Luciole

        chemin = "../Visuels/décorations/feuilles"

        self.x_min = -largeur_ecran
        self.x_max = largeur_niveau + largeur_ecran

        self.sprites_mouvement = pg.sprite.Group()
        self.hauteur_niveau = hauteur_niveau


        for feuille in range(nb_feuilles):
            x = randint(self.x_min,self.x_max)
            y = randint(-self.hauteur_niveau,-10) # Pour apparaître à des hauteurs différentes sur l'écran, on crée les sprites à des hauteurs différentes en dehors de l'écran
            sprite = TuileAnimée(0,(x,y),chemin)
            groupe.add(sprite)
            self.sprites_mouvement.add(sprite)

    def mouvement(self):
        '''
        Génère le déplacement des feuilles vers le bas et les téléporte au-dessus du niveau si leur rectangle dépasse la hauteur du niveau
        '''
        for sprite in self.sprites_mouvement.sprites():
            sprite.rect.y += 2
            if sprite.rect.y > self.hauteur_niveau + 10: 
                x = randint(self.x_min,self.x_max)
                y = randint(-self.hauteur_niveau,-10)
                sprite.rect.topleft = (x,y)

           

    




        

