# Importation
import pygame as pg
from sys import exit
from parametres import *
from niveau import Niveau
from infos_jeu import *

# Il vaut mieux lire d'abord ce fichier, puis le fichier niveau, le fichier importation, le fichier tuiles, 
# le fichier joueur puis le reste dans cet ordre pour faciliter la compréhension.

class Jeu:
    '''
    Classe qui gère la succession des niveaux par une liste dont on prend une des valeurs selon un index augmenté de 1 à chaque fin de niveau.
    La classe compte aussi le nombre de KO, le temps de complétion et le nombre de cristaux collectés pour les afficher à l'écran de fin. 
    '''
    def __init__(self):
        self.liste_niveaux = [niveau_1,niveau_2,niveau_3,niveau_4,niveau_5,niveau_6,fin] # Liste modifiable si l'on veut tester un seul niveau
        self.niveau_max = 0
        self.info_niveau = self.liste_niveaux[self.niveau_max]
        self.niveau = Niveau(self.info_niveau,ecran,self.niveau_max,True) # Voir le fichier niveau pour plus d'infos
        self.nb_KO = 0
        self.nb_collectables = 0
        self.tps_total = 0
        
    def fonctionnement(self):
        '''
        Fonction qui sera placée sous la boucle while nécessaire au fonctionnement du jeu. On y fait appel à toutes les autre fonctions de la classe Jeu.
        '''

        # Parcourt l'ensemble des événements en cours dans le jeu et stoppe le programme si l'on appuie sur la croix de fermeture.
        for event in pg.event.get():
            if event.type == pg.QUIT:
                pg.quit()
                exit()  

        ecran.fill("black")
        ecran.blit(self.niveau.arriere_plan,(0,0)) # blit : afficher une image sur l'écran, ici effectué en premier pour servir de fond d'écran.
                
        self.choix_niveau()
        
        self.niveau.disposition_niveau() 

        if self.niveau.ecran_fin: self.affichage_fin()
            

        pg.display.update()

    def choix_niveau(self):
        '''
        Procède au passage au niveau suivant à la fin de celui qui est en cours. Toutes les mesures propres au niveau (KO,cristal,temps) sont ajoutées aux mesures générales du jeu.
        L'index de niveau max étant incrémenté de 1, on prend la prochaine valeur dans la liste et on redéfinit le niveau à partir du nouveau dictionnaire.
        Les booléens ont pour fonction: activation du fondu et inversion du fondu
        '''
        if self.niveau.niveau_max > self.niveau.niveau_en_cours and not self.niveau.changement_niveau: 

            self.nb_KO += self.niveau.compteur_mort
            if self.niveau.compteur_collectable: self.nb_collectables += 1
            self.tps_total += self.niveau.temps_en_sec

            self.niveau_max = self.niveau.niveau_max
            self.info_niveau = self.liste_niveaux[self.niveau_max]
            self.niveau = Niveau(self.info_niveau,ecran,self.niveau_max)

            self.niveau.niveau_change = True
            self.niveau.jouer = True

    
    def affichage_fin(self):
        '''
        Définit les trois textes qui vont être affichés successivement sur l'écran à la fin du jeu.
        Le premier if permet d'actualiser la valeur de temps total puis de la convertir en heures, minutes et secondes si la valeur n'est pas déjà une liste.
        Le reste détermine ce qui est affiché en fonction du nombre de fois que le joueur a cliqué avec la souris. 
        Pour les différentes fonctions utilisées, voir les fichiers niveau (affichage) et tuiles (Bouton.clic())
        '''

        ecran.fill("black")
        if not isinstance(self.tps_total,list): 
            self.tps_total += self.niveau.temps_en_sec
            self.tps_total = [int(self.tps_total / 3600),int(self.tps_total % 3600 // 60),int(self.tps_total) % 3600 % 60,int(self.tps_total % 1 * 1000)]

            # On place un zéro devant les minutes et/ou les secondes si leur valeur est inférieure à 10.
            if self.tps_total[1] < 10: self.tps_total[1] = "0" + str(self.tps_total[1])
            if self.tps_total[2] < 10: self.tps_total[2] = "0" + str(self.tps_total[2])

        if self.niveau.index_texte == 0:
            self.niveau.affichage_texte(self.niveau.texte_fin,0,0,False)
        elif self.niveau.index_texte == 1:
            self.niveau.affichage_statistiques(self.nb_KO,self.nb_collectables,self.tps_total)
        else:
            self.niveau.bouton_quitter_jeu.affichage()
            if self.niveau.bouton_quitter_jeu.clic():
                pg.quit()
                exit()

        self.niveau.clic()

    


# Mise en place de pygame
pg.init()
ecran = pg.display.set_mode((largeur_ecran,hauteur_ecran))
pg.display.set_caption("The Elder's Heart") # Change nom du bandeau en haut de la fenêtre de Pygame
img_psec = pg.time.Clock() # Classe qui sert à fixer le nombre d'images par seconde
jeu = Jeu()

while True:
    # Boucle while permettant de faire tourner le jeu tant que pg.quit() n'est pas utilisé

    jeu.fonctionnement()
    
    img_psec.tick(60) # Il y aura donc 60 images par seconde