# Importation
from csv import reader # Fonction pour lire les fichiers .csv pour la map
from parametres import taille_case
from os import walk 
# walk permet de donner le chemin vers un dossier, son nom et le nom de chaque fichier qu'il contient (renvoie un tuple avec le chemin, les dossiers contenus et les fichiers)
import pygame as pg



def importation_dossier(chemin):
    '''
    Permet d'importer les images d'un dossier sans manuellement les importer une par une.
    Un chemin vers chaque image est créé, puis une surface est générée grâce à ce chemin et ajoutée dans une liste de surfaces.
    '''
    
    liste_surfaces = []

    for _,__,fichiers_images in walk(chemin): # les enderscores permettent d'indiquer à Python que les deux premières listes du tuple ne seront pas utilisées
        fichiers_images.sort() # Je range la liste au cas où les images n'arrivent pas dans le bon ordre
        for image in fichiers_images:
            chemin_complet = chemin + "/" + image # Rajoute une étape dans le chemin pour arriver à l'image voulue
            surf_image = pg.image.load(chemin_complet).convert_alpha()
            liste_surfaces.append(surf_image)

    return liste_surfaces

def importation_csv(chemin):
    '''
    Importe un fichier .csv à partir d'un chemin donné (que l'on obtient dans infos_jeu)
    '''

    map_terrain = []

    with open(chemin) as map: # map est une variable dans laquelle on va stocker les fichiers .csv
        # with facilite la manipulation des fichiers, permet de le fermer tout seul à la fin de la fonction
        niveau = reader(map,delimiter = ",") # La fonction reader() lit le fichier .csv en comptant les virgules comme objet séparant les différentes valeurs (séparation en règle dans une liste)
        for ligne in niveau:
            # Création d'une liste contenant des sous-listes correspondant aux lignes de haut en bas de la map
            map_terrain.append(list(ligne)) 
            
        return map_terrain

def importation_tuiles_coupees(chemin):
    '''
    Permet, à partir d'une image dont les dimensions sont des multiples de la taille des cases, de créer un ensemble de tuiles en 64 par 64.
    '''
    surface = pg.image.load(chemin).convert_alpha()
    nb_tuiles_x = surface.get_size()[0] // taille_case # get_size renvoie un tuple avec les dimensions de l'image, que l'on divise par la taille des cases afin d'en avoir le nombre
    nb_tuiles_y = surface.get_size()[1] // taille_case

    tuiles_coupees = []
    # Processus très semblable à l'affichage des tuiles sur l'écran, sauf que la fonction enumerate() n'est pas nécessaire
    for ligne in range(nb_tuiles_y):
        for colonne in range(nb_tuiles_x):
            # Donne les coordonnées du point en haut à gauche de chacune des tuiles contenues dans l'image
            x = colonne * taille_case
            y = ligne * taille_case

            # Création d'une nouvelle surface sur laquelle va être affichée une partie en 64 × 64 de l'image de base
            # le deuxième argument permet de rendre invisibles tous les pixels non utilisés sur la surface
            nvle_surf = pg.Surface((taille_case,taille_case), flags = pg.SRCALPHA) 
            
            # Affichage d'un rectangle de 64 × 64 sur la nouvelle surface, avec les coordonnées calculées plus haut
            nvle_surf.blit(surface,(0,0),pg.Rect(x,y,taille_case,taille_case)) 

            tuiles_coupees.append(nvle_surf)
    
    return tuiles_coupees


