# Importation
import pygame as pg
from importation import importation_dossier
from tuiles import FeuilleSaut

class Joueur(pg.sprite.Sprite):
    '''
    Sprite contrôlé par le joueur. Y sont associés toutes les méthodes de mouvement, des interactions avec l'environnement, la physique...
    '''
    def __init__(self,pos,groupe_collisions,groupe_KO,groupe_rbd,groupe_pltf_cass,ds_actif = False):
        super().__init__()
        
        self.importation_statut()

        # Animations
        self.index_images = 0
        self.vitesse_animation = 0.1
        self.image = self.animations["sur_place"][self.index_images]
        self.rect = self.image.get_rect(topleft = pos)

        # Mouvement du joueur
        self.rect_collisions = pg.rect.Rect(pos[0] + 8,pos[1] + 12,28,50) # Rectangle plus petit que celui de l'image pour les collisions avec les autres éléments
        self.direction = pg.math.Vector2() # Donne une liste avec une position x et y, correspondant à un vecteur utilisable pour les déplacements
        self.vitesse = 6.8 # Pour les déplacements horizontaux
        self.gravite = 0.6 # Voir plus bas la méthode applic_gravite()
        self.vitesse_saut = 3 # Voir pour les deux variables la méthode entree_touches()
        self.accel = 0.07

        # Groupes pour les interactions
        self.groupe_collisions = groupe_collisions
        self.groupe_KO = groupe_KO
        self.groupe_rbd = groupe_rbd
        self.groupe_pltf_cass = groupe_pltf_cass

        # Statut du joueur
        self.pos = pos
        self.statut = "sur_place"
        self.jeu_enc = True # Booléen pour gérer l'état du jeu (en cours/pause)
        
        self.orientation_droite = True # Pour l'inversion de l'image selon l'orientation du joueur

        # Booléen indispensable pour de nombreux usages. Il est True si le joueur a des collisions verticales.
        self.sur_sol = False

        # Booléens pour le saut
        self.a_saute = False
        self.compteur_saut = False
        self.en_chute = False
        self.ds_actif = ds_actif # active ou non le double saut
        self.feuille = False
        self.en_rebond = False

        # Active la réapparition du joueur
        self.KO = False

    def importation_statut(self):
        ''' Méthode similaire à celle de TuileAnimeePlus'''
        chemin_fichier_joueur =  "../Visuels/joueur/"
        self.animations = {"sur_place" : [],"course" : [],"saut" : [],"chute" : []} 
        
        for animation in self.animations.keys():
            chemin_complet = chemin_fichier_joueur + animation
            self.animations[animation] = importation_dossier(chemin_complet)

    def animation(self):
        ''' 
        Tous les principes ont déjà été vus dans le fichier tuiles : animations avec plusieurs sets, inversion d'image, repositionnement au sol.
        '''
        animation = self.animations[self.statut]

        # Réinitialisation de l'index des images
        self.index_images += self.vitesse_animation
        if self.index_images >= len(animation): self.index_images = 0

        image = animation[int(self.index_images)]

        if self.orientation_droite: 
            self.image = image 
            self.rect.bottomleft = self.rect_collisions.bottomleft # Permet au rectangle de l'image de suivre le rectangle de collisions
        else: 
            self.image = pg.transform.flip(image,True,False) 
            self.rect.bottomright = self.rect_collisions.bottomright # Idem

        self.rect = self.image.get_rect(midbottom = self.rect.midbottom)

 
    def entree_touches(self):
        '''
        Enregistrement des touches pressées par le joueur et réactions en fonction.
        Le déplacement réel du joueur se fait dans update() et applic_gravite().
        '''
        touches = pg.key.get_pressed() # Liste de booléens correspondant aux touches du clavier, True si elles sont pressées

        # Déplacement horizontal
        if touches[pg.K_d]: # Par exemple, si la touche d est pressée, le joueur va vers la droite.
            # Tant que la première composante du vecteur est inférieure à 1, on y ajoute la valeur d'accélération. Simule une accélération.
            if self.direction.x < 1: self.direction.x += self.accel 
            self.orientation_droite = True
            if touches[pg.K_q]: self.direction.x = 0 # Réinitialise la vitesse si le joueur appuie sur la touche opposée en même temps.
        elif touches[pg.K_q]: 
            # Même principe.
            if self.direction.x > -1: self.direction.x -= self.accel
            self.orientation_droite = False
            if touches[pg.K_d]: self.direction.x = 0
        else: self.direction.x = 0 # Autrement, la première composante est nulle: le joueur ne se déplace pas.

        # Saut
        if touches[pg.K_SPACE] and self.sur_sol and not self.compteur_saut: 
            # Active le saut réalisé depuis le sol si le joueur presse la barre espace
            self.a_saute = True 
            self.compteur_saut = True

        if touches[pg.K_SPACE] and self.a_saute and self.vitesse_saut < 10.5: 
            # Système d'accélération similaire au déplacement horizontal. 
            self.vitesse_saut += 0.8
            self.direction.y = - self.vitesse_saut # Comme le joueur doit se déplacer vers le haut, la valeur de la deuxième composante doit être négative.

        if not touches[pg.K_SPACE] and self.a_saute:
            # Si l'on lâche la barre espace quand on est en l'air (a_saute), on active un nouveau booléen qui permet de faire un double saut.
            self.a_saute = False
            self.en_chute = True
            self.vitesse_saut = 3
        
        if not touches[pg.K_SPACE] and self.compteur_saut and self.sur_sol: self.compteur_saut = False # Permet d'empêcher de resauter si l'on garde la barre espace appuyée en tombant au sol

        if self.ds_actif:
            # Double saut
            if self.en_chute and touches[pg.K_SPACE] and not self.sur_sol and self.vitesse_saut == 3: 
                # Si le joueur est en l'air, qu'il touche la barre espace et que la vitesse de saut a bien été réalisée, il effectue un double saut.
                # Les booléens nécessaires sont réinitialisés
                self.feuille = True # Active le sprite de la feuille enchantée de saut dans la méthode disposition_niveau() de Niveau.
                self.direction.y = - 4 * self.vitesse_saut
                self.en_chute = False
                self.compteur_saut = True

        # Met le jeu en arrêt => menu pause ou lecture d'un dialogue
        # Je le mets ici car je dispose déjà de la liste des touches entrées
        if touches[pg.K_ESCAPE] and self.jeu_enc: self.jeu_enc = False 



    
    def obtention_statut(self):
        '''
        Permet d'obtenir le statut du joueur dépendant de certaines conditions
        '''
        if self.direction.y < 0: 
            # Si le joueur va vers le haut => il saute
            self.vitesse_animation = 0.07
            self.statut = "saut"
        elif self.direction.y > 1:
            # Lorsque le personnage est au sol, la gravité est soit égale à 0, soit à 0.85 (quand elle est à 0.85, il est replacé au-dessus du terrain et elle est remise à 0)
            # Il faut donc que la deuxième composante du vecteur soit supérieure à 1 pour qu'on choisisse l'animation de chute
            self.statut = "chute"
        
        else:
            if self.direction.x == 0: 
                # Si le joueur ne bouge pas et est au sol
                self.vitesse_animation = 0.1
                self.statut = "sur_place"
                
            else: 
                self.vitesse_animation = 0.15
                self.statut = "course"

    def collisions_horizontales(self):
        '''
        Méthode gérant séparément les collisions horizontales.
        '''
        for sprite in self.groupe_collisions.sprites(): # Parcourt la liste de sprites présents dans le groupe des sprites de collision
            if sprite.rect.colliderect(self.rect_collisions): # Vérifie la collision d'un rectangle de sprite avec celui du joueur
                if sprite in self.groupe_KO: self.KO = True # Si le sprite est dans le bon groupe, la variable KO est mise à True => réapparition
                if self.direction.x < 0:
                    # Replacement si le joueur arrive par la droite : la gauche du joueur est constamment replacée à l'extrémité droite du sprite
                    # Il y a quelques problèmes de collision avec les champis rebondissants lorsque le set d'animation change (et donc le re)
                    if sprite not in self.groupe_rbd or 0 <= self.direction.y < 1: self.rect_collisions.left = sprite.rect.right 
                if self.direction.x > 0:
                    # Idem mais par la gauche
                    if sprite not in self.groupe_rbd or 0 <= self.direction.y < 1: self.rect_collisions.right = sprite.rect.left 



    def collisions_verticales(self):
        '''
        Méthode gérant séparément les collisions verticales. 
        Elle est nettement plus importante que l'autre car beaucoup de booléens et d'interactions y sont liés.
        '''
        if self.direction.y > 1 or self.direction.y < 0: self.en_rebond = False # Booléen nécessaire pour supprimer la feuille de saut lorsque le joueur rebondit sur un champignon
        for sprite in self.groupe_collisions.sprites(): 
            if self.KO and sprite in self.groupe_pltf_cass: sprite.temps_attente = -1 # Réinitialisation des sprites de plateformes cassantes si le joueur est mis KO.
            if sprite.rect.colliderect(self.rect_collisions):
                if sprite in self.groupe_KO: self.KO = True # Même principe que pour les collisions horizontales.
                if self.direction.y > 0:
                
                    # Réinitialisation des variables pour le saut
                    self.a_saute = False
                    self.vitesse_saut = 3

                    self.en_chute = True # Le joueur peut double sauter après être tombé d'une plateforme sans sauter

                    if sprite in self.groupe_rbd: 
                        # Interaction avec les champignons rebondissants
                        self.direction.y = -20 # Rebondit plus haut que le saut de base
                        sprite.index_images = 0 # Réinitialisation de l'index et changement de statut
                        sprite.statut = "utilisé"
                        self.en_rebond = True # La feuille de saut disparaîtra 

                    else: 
                        # Replacement si le joueur arrive par le haut et qu'il touche le sol
                        self.direction.y = 0
                        self.sur_sol = True
                        self.rect_collisions.bottom = sprite.rect.top 
                        if sprite in self.groupe_pltf_cass and sprite.statut == "normal": sprite.statut = "émiettement" # Interaction avec la plateforme cassante
                            
                    
                if self.direction.y < 0 and sprite not in self.groupe_rbd:
                    # Replacement si le joueur arrive par le bas et qu'il touche le plafond
                    self.rect_collisions.top = sprite.rect.bottom  
                    self.direction.y = 0

        if self.sur_sol and self.direction.y < 0 or self.direction.y > 1:
            # Si le joueur est en l'air (saut ou chute), la variable sur_sol est remise à False
            self.sur_sol = False
        
        
        



    def application_gravité(self):
        # Simulation de gravité : la gravité est ajoutée à la deuxième composante du vecteur à chaque frame et abaisse le rectangle du joueur
        # Deuxième instruction : permet à la fois d'effectuer le saut et la chute.
        self.direction.y += self.gravite
        self.rect_collisions.y += self.direction.y

    def update(self):
        if self.jeu_enc:
            # Si le jeu n'est pas en pause, les déplacements sont activés
            self.entree_touches()
            self.rect_collisions.x += self.direction.x * self.vitesse # Le rectangle du joueur se déplace selon la première composante du vecteur multipliée par une valeur de vitesse.
        self.obtention_statut()
        self.animation()
        self.collisions_horizontales()
        self.application_gravité()
        self.collisions_verticales()


