# Importation
import pygame as pg
from importation import importation_csv,importation_tuiles_coupees
from parametres import taille_case, hauteur_ecran,limites_camera, largeur_ecran
from tuiles import *
from joueur import Joueur
from decoration import Lucioles, Feuilles
from infos_jeu import *

class Niveau:
    '''
    Classe la plus importante de tout le jeu. 
    Elle en gère la majorité des aspects : textures, transitions, état du jeu (en cours, pause, menu de départ), différentes mesures, textes, caméra.
    '''
    def __init__(self,info_niveau,surface_affichage,niveau_max,menu = False):
        # Mise en place générale
        self.info_niveau = info_niveau
        self.surface_affichage = surface_affichage
        self.arriere_plan = pg.image.load(info_niveau["fond"]).convert_alpha() # convert_alpha() permet à Pygame de mieux gérer l'image
        self.assombr_fin = pg.image.load("../Visuels/fonds/assombr_fin.png").convert_alpha() # Rectangle d'assombrissement actif dans le dernier niveau

        # Partie du jeu, booléen utilisé plus tard (notamment pour l'activation du double saut)
        if self.info_niveau in [test,niveau_4,niveau_5,niveau_6,fin]: self.deuxieme_partie = True
        else: self.deuxieme_partie = False

        # Groupes : deux groupes communs pour la caméra, des groupes particuliers pour les sprites avec interactions avec le joueur (autres que collisions) et des groupes pour des sprites uniques
        self.groupe_sprites = GroupeCamera()
        self.groupe_vegetation = GroupeCamera() # Groupe spécifique à la végétation, utilisé pour afficher les textures concernées au-dessus des autres

        self.groupe_collisions = pg.sprite.Group()
        self.groupe_check = pg.sprite.Group()
        self.groupe_casse = pg.sprite.Group()
        self.groupe_KO = pg.sprite.Group()
        self.groupe_rbd = pg.sprite.Group()

        self.groupe_arrivee = pg.sprite.GroupSingle()
        self.groupe_feuille_saut = pg.sprite.GroupSingle()
        self.groupe_collectable = pg.sprite.GroupSingle()
        self.groupe_potion = pg.sprite.GroupSingle()
        self.groupe_lettre = pg.sprite.GroupSingle()
        
        # Variables pour les changements de niveau. Une méthode associée au sprite Portail permet d'incrémenter le niveau max et ainsi de changer de niveau
        # Les booléens sont utilisés pour la méthode fondu() qui gère les apparitions du joueur, voir plus bas
        self.niveau_max = niveau_max
        self.niveau_en_cours = self.niveau_max
        self.changement_niveau = False
        self.niveau_change = False

        # Variables pour mesurer le temps, compter le nombre de KO et vérifier ou non que le joueur a récupéré le cristal à collecter.
        self.compteur_mort = 0
        self.compteur_collectable = False

        self.temps_completion = 0
        self.temps_en_sec = 0
        self.temps_depart = 0
        self.temps_passe = 0


        # Mise en place des boutons, voir la classe en question dans tuiles pour plus d'infos
        self.bouton_reprendre = Bouton(0,(largeur_ecran // 2,hauteur_ecran // 2 - 100),"../Visuels/boutons/btn_reprendre.png",self.surface_affichage)
        self.bouton_quitter = Bouton(0,(largeur_ecran // 2,hauteur_ecran - 280),"../Visuels/boutons/btn_quitter.png",self.surface_affichage)
        self.bouton_quitter_jeu = Bouton(0,(largeur_ecran // 2,hauteur_ecran //2),"../Visuels/boutons/btn_quitter.png",self.surface_affichage)
        self.bouton_jouer = Bouton(0,(largeur_ecran // 2,hauteur_ecran // 2 - 100),"../Visuels/boutons/btn_jouer.png",self.surface_affichage)
        self.bouton_commandes = Bouton(0,(largeur_ecran // 2,hauteur_ecran - 280),"../Visuels/boutons/btn_cmds.png",self.surface_affichage)
        self.bouton_retour = Bouton(0,(96,96),"../Visuels/boutons/btn_retour.png",self.surface_affichage)

        # Booléens nécessaires pour le fondu() et pour l'affichage des boutons, rectangle d'assombrissement pour le menu pause
        self.quitter = False
        self.jouer = False

        self.commandes = False
        self.menu = menu
        
        self.assombrissement = pg.image.load("../Visuels/fonds/assombrissement.png").convert_alpha()

        # MISE EN PLACE DU TEXTE
        # La méthode textes() est expliquée plus bas.
        # On crée ici des images de texte à partir d'une classe Font (pour la police) et de chaînes de caractères

        self.police = pg.font.Font("../Police/Alkhemikal.ttf",55)

        self.index_texte = 0 # Permet de sélectionner une liste de chaîne de caractères dans une liste en contenant plusieurs (liste pour dialogues particulièrement)

        # Boites transparentes accompagnées de leur rectangle obligatoire, affichées en fond derrière le texte
        self.boite_cmd = pg.image.load("../Visuels/boutons/boite_cmds.png").convert_alpha()
        self.rect_boite_cmd = self.boite_cmd.get_rect(center = (largeur_ecran // 2,hauteur_ecran // 2))

        self.boite_dialogue = pg.image.load("../Visuels/boite_dlg.png").convert_alpha()
        self.rect_boite_dialogue = self.boite_dialogue.get_rect(center = (largeur_ecran // 2,640))
        
        # Textes pour l'écran de fin
        contenu_texte_fin = ["Grâce à la potion, votre frère est sorti","sain et sauf de sa grave maladie.","Quant à vous, vous êtes devenu un arbre adolescent !"]
        self.texte_fin = self.textes(contenu_texte_fin,300,80)
        self.ecran_fin = False # Permet d'activer l'écrand e fin lorsqu'il est mis à True

        # Création d'une liste de groupes de textes à partir d'une liste contenant des listes de chaînes de caractères
        liste_texte_lettre = [
            ["Mon enfant,","Comme je ne pouvais pas être là pour te", "donner des conseils en personne,"],
            ["voici une lettre que je te dédie,","remplie de précieux conseils pour ton aventure."],
            ["Comme tu le sais, ton frère est très mal en point,","et il faut que tu ailles chercher une potion","pleine d'essence de l'Arbre pour le soigner."],
            ["Tu pourras aussi en profiter pour réaliser ta","formation d'arbre à part entière."],
            ["Tu as probablement déjà lu mon livret de","commandes. Un conseil pour le saut :","tu peux sauter à des hauteurs différentes."],
            ["Pour un petit saut, frôle la barre espace,","pour un grand, laisse appuyé."],
            ["Aussi, je te conseille de toujours avancer","quand tu sautes, et de ne plus toucher à rien","si tu veux t'arrêter en l'air sur une plateforme."],
            ["Fais attention aux ronces et épines","dehors ! Tu risques de te faire très mal sinon.","Tu le sais, ton bois est encore tendre."],
            ["Tu trouveras des balises sur ton chemin","auxquelles tu retourneras s'il t'arrive malheur.","Enfin, j'ai entendu parler de chemins alternatifs,"],
            ["qui sont plus difficiles que les autres.","Si tu en repères, peut-être qu'il y aura","une récompense à la fin, qui sait..."],
            ["Oh, et j'oublie ! Tu trouveras sans doute sur","ton trajet une feuille enchantée qui te","permettra de sauter une deuxième fois en l'air."],
            ["Trève de tergiversations, il est l'heure","pour toi d'y aller. Bon voyage","et surtout bon courage ! Tu peux le faire !"]
            ]
        self.textes_lettre = []
        for liste_texte in liste_texte_lettre:
            self.textes_lettre.append(self.textes(liste_texte,600,40))

        self.passage_texte = False 
        self.action = False


        # Variables pour le blackout (voir plus bas la fonction reapparition) 
        # le booléen permet d'enregistrer s'il y a eu inversion et la variable est une liste de surfaces que l'on parcourt avec l'index
        self.vitesse_animation = 0.22
        self.index_image = 0
        self.inversion = False
        self.blackout = importation_dossier("../Visuels/blackout")



        # TEXTURES

        # Position de départ, modifiée au contact d'un checkpoint
        self.pos_dep = (0,0)

        # Liste des textures spécifiques à différents types de niveaux
        # La place de chaque string dans la liste est important car elle détermine l'ordre d'affichage et donc de superposition des images sur l'écran
        liste_textures_gen = ["herbe","lianes","joueur"]
        liste_textures_niveaux = ["checkpoints","ronces","collectable","portail"]
        liste_textures_1P = ["arbres","terre"]
        liste_textures_2P = ["champis","racines"]


        # Mise en place : elle se fait en fonction du dictionnaire utilisé, correspondant au niveau en cours. Voir pour cela le fichier infos_jeu.
        # Le principe est que l'on crée un tableau de valeurs à partir des fichiers csv créés sur Tiled et importés grâce à la fonction importation_csv (voir le fichier importation)
        # et que l'on place les textures sur l'écran selon la valeur présente à un certain endroit. Je précise plus bas dans la fonction addition_groupe().
        # Comme dans les listes précédentes, l'ordre d'appel de la méthode addition_groupe() est important car déterminant quelles textures vont être affichées en premier.
        for texture in liste_textures_gen: self.addition_groupe(importation_csv(info_niveau[texture]),texture)


        if not self.deuxieme_partie:
            for texture in liste_textures_1P: self.addition_groupe(importation_csv(info_niveau[texture]),texture)
        else:
            for texture in liste_textures_2P: self.addition_groupe(importation_csv(info_niveau[texture]),texture)
            if info_niveau != fin: self.addition_groupe(importation_csv(info_niveau["grands_champis"]),"grands_champis")

        if info_niveau != fin: 
            for texture in liste_textures_niveaux: self.addition_groupe(importation_csv(info_niveau[texture]),texture)
        else:
            self.addition_groupe(importation_csv(info_niveau["potion"]),"potion")

        if info_niveau in [niveau_3,niveau_4,niveau_5,niveau_6,test]: 
            self.addition_groupe(importation_csv(info_niveau["pltf_cass"]),"pltf_cass")
            if info_niveau == niveau_3: self.addition_groupe(importation_csv(info_niveau["feuille_saut"]),"feuille_saut")

        if info_niveau == niveau_1: 
            self.addition_groupe(importation_csv(info_niveau["lettre"]),"lettre")
            self.lettre_grand = pg.image.load("../Visuels/lettre/lettre.png") # Image qui sera affichée au moment de la lecture de la lettre.
        

        # Texte dépendant de l'état du joueur
        if self.joueur.ds_actif:
            # Rajoute une commande si le joueur a atteint le niveau 4 et a donc obtenu le double saut
            self.liste_commandes = ["Gauche : q","Droite : d","Saut : barre espace","Double saut : barre espace en l'air","Pause : échap"]
            self.texte_commandes = self.textes(self.liste_commandes,180,100)
        else: 
            self.liste_commandes = ["Gauche : q","Droite : d","Saut : barre espace","Pause : échap"]
            self.texte_commandes = self.textes(self.liste_commandes,240,100)

        # Mise en place de la décoration
        # Voir le fichier decoration pour plus d'informations concernant les classes utilisées
        tableau_valeurs = importation_csv(info_niveau["joueur"]) # On fait cette importation uniquement pour connaître la largeur et la hauteur du niveau
        self.hauteur_niveau = len(tableau_valeurs) * taille_case
        self.largeur_niveau = len(tableau_valeurs[0]) * taille_case

        self.lucioles = Lucioles(self.largeur_niveau,self.hauteur_niveau,info_niveau["lucioles"],self.groupe_sprites)
        if not self.deuxieme_partie: self.feuilles = Feuilles(self.largeur_niveau,self.hauteur_niveau,info_niveau["feuilles"],self.groupe_sprites)

        
    def addition_groupe(self,disposition,type):
        '''
        Fonction servant à ajouter des sprites dans différents groupes ayant divers rôles dans le jeu.
        Elle se fait à partir d'une disposition de valeurs dans un tableau issu d'un fichier csv préalablement 
        importé et d'un type précis de valeur associé à un fichier csv précis.
        On parcourt d'abord chaque sous-liste dans le tableau de valeurs en csv, correspondant aux lignes du niveau
        puis chacune des valeurs à l'intérieur, correspondant aux colonnes.
        Les index que l'on récupère permettent de calculer la position de chaque sprite en multipliant l'index par la taille des cases
        et les valeurs déterminent l'image qui sera affichée (et éventuellement le type de sprite)
        '''

        for index_ligne,ligne in enumerate(disposition):
            for index_col,valeur in enumerate(ligne):
                
                if valeur != "-1":
                    # Si la valeur vaut -1, aucune surface n'est ajoutée dans un groupe.
                    # On multiplie donc l'index de la ligne et celui de la colonne par 64 pour obtenir les vraies coordonnées sur la carte
                    x = index_col * taille_case
                    y = index_ligne * taille_case

                    # Création 
                    if type == "herbe":
                        # Liste de tuiles en 64 × 64 découpées à partir d'une image entière. Voir importation pour plus d'infos.
                        liste_tuile_terrain = importation_tuiles_coupees("../Visuels/terrain/herbe.png")
                        surf_tuile = liste_tuile_terrain[int(valeur)] # On prend une certaine surface dans la liste selon la valeur
                        sprite = TuileImmobile(taille_case,(x,y),surf_tuile)
                        self.groupe_vegetation.add(sprite)

                    elif type == "lianes":
                        liste_tuile_terrain = importation_tuiles_coupees("../Visuels/terrain/lianes.png")
                        surf_tuile = liste_tuile_terrain[int(valeur)] 
                        sprite = TuileImmobile(taille_case,(x,y),surf_tuile)
                        self.groupe_vegetation.add(sprite)

                    elif type == "joueur":
                        # La position de départ est définie à la position initiale du joueur
                        # Voir le fichier kuuki pour comprendre le sens des arguments de la classe
                        self.pos_dep = (x,y)
                        if self.deuxieme_partie: self.joueur = Joueur(self.pos_dep,self.groupe_collisions,self.groupe_KO,self.groupe_rbd,self.groupe_casse,True)
                        else: self.joueur = Joueur(self.pos_dep,self.groupe_collisions,self.groupe_KO,self.groupe_rbd,self.groupe_casse)
                        if not self.menu: self.groupe_sprites.add(self.joueur) # Tant que le menu est activé, le joueur ne doit apparaître sur l'écran => on ne le met pas dans le groupe d'affichage
                        
                    else:

                        if type == "terre" or type == "racines" or type == "ronces":
                            if type == "terre": liste_tuile_terrain = importation_tuiles_coupees("../Visuels/terrain/terre.png")
                            elif type == "racines": liste_tuile_terrain = importation_tuiles_coupees("../Visuels/terrain/racines.png")
                            else: liste_tuile_terrain = importation_tuiles_coupees("../Visuels/terrain/ronces.png")

                            surf_tuile = liste_tuile_terrain[int(valeur)] 
                            sprite = TuileImmobile(taille_case,(x,y),surf_tuile)
                            self.groupe_collisions.add(sprite)
                            if type == "ronces": self.groupe_KO.add(sprite)

                        if type == "arbres":
                            if valeur == "0": sprite = Arbre(0,(x,y),"../Visuels/terrain/petit_arbre",40)
                            elif valeur == "1": sprite = Arbre(0,(x,y),"../Visuels/terrain/moyen_arbre",70)
                            elif valeur == "2": sprite = Arbre(0,(x,y),"../Visuels/terrain/grand_arbre",210)
                            self.groupe_collisions.add(sprite)

                        if type == "champis":
                            if valeur == "0": sprite = Champis(0,(x,y),"../Visuels/terrain/champis_imm/or1.png",10)
                            elif valeur == "1": sprite = Champis(0,(x,y),"../Visuels/terrain/champis_imm/vrt1.png",-20)
                            elif valeur == "2": sprite = Champis(0,(x,y),"../Visuels/terrain/champis_imm/ro1.png",-20)
                            elif valeur == "3": sprite = Champis(0,(x,y),"../Visuels/terrain/champis_imm/or2.png",10)
                            elif valeur == "4": sprite = Champis(0,(x,y),"../Visuels/terrain/champis_imm/vrt2.png",-20)
                            elif valeur == "5": sprite = Champis(0,(x,y),"../Visuels/terrain/champis_imm/ro2.png",-20)
                            elif valeur == "6": sprite = Champis(0,(x,y),"../Visuels/terrain/champis_imm/bl2.png",10)
                            elif valeur == "7": sprite = Champis(0,(x,y),"../Visuels/terrain/champis_imm/bl1.png",10)
                            elif valeur == "8": 
                                sprite = ChampiRebond((x,y))
                                self.groupe_rbd.add(sprite)
                            self.groupe_collisions.add(sprite)

                        if type == "grands_champis":
                            if valeur == "8": sprite = Champis(0,(x,y),"../Visuels/terrain/grands_champis/or.png",0)
                            elif valeur == "9": sprite = Champis(0,(x,y),"../Visuels/terrain/grands_champis/vrt.png",0)
                            elif valeur == "10": sprite = Champis(0,(x,y),"../Visuels/terrain/grands_champis/tronc.png",0)
                            elif valeur == "11": sprite = Champis(0,(x,y),"../Visuels/terrain/grands_champis/pied.png",0)
                            self.groupe_collisions.add(sprite)

                        if type == "checkpoints":
                            sprite = Checkpoint((x,y))
                            self.groupe_check.add(sprite)


                        if type == "pltf_cass":
                            sprite = PlateformeCassante((x,y))
                            self.groupe_casse.add(sprite)
                            self.groupe_collisions.add(sprite)

                        if type == "portail":
                            if valeur == "0": sprite = Portail(0,(x,y),"../Visuels/portail",self.niveau_max)
                            elif valeur == "1": 
                                sprite = EntreeArbre(0,(x,y),"../Visuels/porte_arbre",self.niveau_max)
                                self.groupe_collisions.add(sprite)
                            self.groupe_arrivee.add(sprite)

                        if type == "feuille_saut":
                            sprite = FeuilleSautSurplace((x,y),self.groupe_sprites)
                            self.feuille_saut = sprite
                            self.groupe_feuille_saut.add(sprite)

                        if type == "collectable":
                            sprite = Collectable((x,y),self.groupe_sprites)
                            self.groupe_collectable.add(sprite)

                        if type == "lettre":
                            sprite = Lettre(0,(x,y),"../Visuels/lettre/animation")
                            self.lettre = sprite
                            self.groupe_lettre.add(sprite)

                        if type == "potion":
                            sprite = Potion(0,(x,y),"../Visuels/potion")
                            self.potion = sprite
                            self.groupe_potion.add(sprite)


                        self.groupe_sprites.add(sprite)

    
    def fondu(self):
        '''
        Crée une animation de fondu si certains booléens sont activés. 
        Le temps que l'animation se fasse, le joueur est retiré du groupe_sprites afin d'empêcher tout mouvement
        '''
        # Au début, j'ai voulu immobiliser le joueur le temps que l'animation de mort se fasse, mais cela engendrait trop de problèmes, visuels et techniques.
        # On retire alors le joueur du groupe des sprites actualisés le temps de l'animation. C'est plus brutal mais ça fonctionne.
        
        if self.joueur.KO or self.quitter or self.jouer or self.changement_niveau: 
            self.groupe_sprites.remove(self.joueur)

            if self.jouer and self.niveau_change: 
                # On passe directement à la deuxième partie de l'animation après un changement de niveau (juste après que le niveau a été initialisé)
                self.blackout.reverse()
                self.inversion = True
                self.niveau_change = False
                self.temps_depart = pg.time.get_ticks() # Permet de réinitialiser temps_passe en changeant de niveau. Voir le fonctionnement de mesure du temps plus bas.
            
            # Instructions basiques d'animation : affichage d'une image choisie dans une liste à partir d'un index que l'on incrémente progressivement à chaque frame et dont on prend la partie entière
            self.surface_affichage.blit(self.blackout[int(self.index_image)],(0,0))
            self.index_image += self.vitesse_animation

            if self.index_image > len(self.blackout) and not self.inversion:

                # Permet de cacher l'absence du joueur pendant la deuxième partie de l'animation en plaçant le rectangle de la caméra dans la zone noire de l'animation
                # Rend également indépendant de la position du joueur le repositionnement de la caméra
                # Le replacement diffère selon l'ordonnée de la position de départ pour éviter de voir le vide au-delà des limites du niveau
                if 0 < self.pos_dep[1] < self.hauteur_niveau // 2:
                    self.groupe_sprites.rect_camera.topleft = self.pos_dep 
                    self.groupe_vegetation.rect_camera.topleft = self.pos_dep
                elif self.pos_dep[1] > self.hauteur_niveau - 130:
                    self.groupe_sprites.rect_camera.bottomleft = self.pos_dep[0],self.pos_dep[1] - 155
                    self.groupe_vegetation.rect_camera.bottomleft = self.pos_dep[0],self.pos_dep[1] - 155
                else:
                    self.groupe_sprites.rect_camera.bottomleft = self.pos_dep [0],self.pos_dep[1] - 30 
                    self.groupe_vegetation.rect_camera.bottomleft = self.pos_dep[0],self.pos_dep[1] - 30

                # Inversion de la liste d'images pour inverser l'animation elle-même
                self.blackout.reverse()
                self.index_image = 0 # Remise à zéro de l'index pour réinitialiser l'animation
                self.joueur.rect_collisions.topleft = self.pos_dep # Téléportation du joueur au dernier checkpoint activé. Voir plus bas et dans le fichier tuile comment cette position est modifiée

                self.inversion = True
                self.changement_niveau = False # Booléen qui était actif après un changement de niveau

                if self.info_niveau == fin:
                    # Si le groupe potion ne contient plus le sprite de type Potion, l'écran de fin est activé.
                    if not self.groupe_potion.has(self.potion): self.ecran_fin = True



            elif self.index_image > len(self.blackout) and self.inversion:
                # Remise à False des booléens d'activation de la fonction, rangement de la liste d'images du fondu et remise à zéro de l'index
                if self.joueur.KO: self.compteur_mort += 1

                if self.joueur.KO or self.jouer: 
                    # Le joueur n'est pas replacé dans le groupe de la caméra si l'on se dirige vers le menu principal
                    if self.deuxieme_partie: self.joueur = Joueur(self.joueur.rect_collisions.topleft,self.groupe_collisions,self.groupe_KO,self.groupe_rbd,self.groupe_casse,True)
                    else: self.joueur = Joueur(self.joueur.rect_collisions.topleft,self.groupe_collisions,self.groupe_KO,self.groupe_rbd,self.groupe_casse)
                    self.groupe_sprites.add(self.joueur)
                    self.jouer = False

                self.quitter = False
                self.inversion = False
                self.blackout.reverse()
                self.index_image = 0

    def menus(self):
        '''
        Fonction qui gère les menus pause et principal. 
        Elle permet entre autres d'afficher des boutons aux rôles différents et de réagir en fonction si l'un des boutons est cliqué
        Voir la tuile Bouton dans le fichier tuiles pour plus d'infos sur les boutons.
        '''
        
        # MENU PAUSE
        if not self.joueur.jeu_enc and not self.passage_texte:  
            # Si le jeu n'est plus en cours et qu'aucun dialogue n'est lancé, un rectangle d'assombrissement est affiché
            self.surface_affichage.blit(self.assombrissement,(0,0))

            self.bouton_reprendre.affichage()
            if not self.joueur.ds_actif or self.info_niveau != niveau_3: self.bouton_quitter.affichage() # La condition empêche le joueur de revenir au menu principal s'il vient d'obtenir le double saut

            if self.bouton_reprendre.clic(): self.joueur.jeu_enc = True
            elif (not self.joueur.ds_actif or self.info_niveau != niveau_3) and self.bouton_quitter.clic(): self.quitter = True

        if self.quitter:
            self.menu = True
            self.commandes = False
            
        # MENU PRINCIPAL
        if self.menu:
            if not self.commandes:
                # Affichage de certains boutons si l'on n'est pas dans le menu de commandes
                self.joueur.jeu_enc = True # Supprime l'assombrissement

                if self.inversion or not self.quitter:
                    # Boutons affichés seulement après la deuxième partie de l'animation de fondu
                    self.bouton_jouer.affichage()
                    self.bouton_commandes.affichage()
                else:
                    # Les boutons correspondant au menu pause sont toujours affichés tant que l'animation de fondu est toujours dans sa première partie
                    self.bouton_reprendre.affichage()
                    self.bouton_quitter.affichage()

                if self.bouton_jouer.clic():
                    self.menu = False
                    self.jouer = True

            else:
                # Affichage du bouton de retour et du texte correspondant aux commandes. Voir plus bas la fonction affichage_texte()
                self.bouton_retour.affichage()
                self.affichage_texte(self.texte_commandes,self.boite_cmd,self.rect_boite_cmd)
                if self.bouton_retour.clic():
                    self.commandes = False

            if self.bouton_commandes.clic():
                self.commandes = True
                

    def textes(self,liste_contenu,pos_y,espacement):
        '''
        Permet de créer des images de texte avec leur rectangle et de leur associer une position sur l'écran puis de les placer dans une liste.
        Les images sont créées à partir d'une police, une chaîne de caractère (le contenu du texte) et une couleur grâce à la méthode render() de la classe Font.
        Le booléen de la méthode désigne l'anti-aliasing, dont je n'ai pas besoin.
        On détermine une position de base pour la ligne la plus haute, et une valeur de décalage pour l'espacement entre chaque ligne.
        '''

        liste_textes = []

        for contenu_texte in liste_contenu:
            pos = (largeur_ecran // 2,pos_y)
            texte = self.police.render(contenu_texte,False,"#2d863b")
            contour = self.police.render(contenu_texte,False,"#0f2e1e")
            rect_texte = texte.get_rect(center = pos)
            # rect_contour = contour.get_rect(center = pos)
            pos_y += espacement


            liste_textes.append((texte,contour,rect_texte))

        return liste_textes

        

    def affichage_texte(self,textes,boite,rect,pres_boite = True):
        '''
        Affiche le texte présent dans la liste créée avec la fonction textes() avec un contour. 
        Le contour est créé en affichant une fois dans chaque coin le même texte mais dans une couleur plus foncée.
        On décale ainsi légèrement la position du texte pour chaque affichage avant de blit en dernier le texte dans la couleur finale.
        Par défaut, il faut donner une surface de boîte de fond et son rectangle, sauf si l'on met pres_boite = False
        '''

        if pres_boite: self.surface_affichage.blit(boite,rect)
        for texte,contour,rect_texte in textes:
            pos = rect_texte.topleft
            self.surface_affichage.blit(contour,(pos[0] - 3, pos[1] - 3))
            self.surface_affichage.blit(contour,(pos[0] - 3, pos[1] +  3))
            self.surface_affichage.blit(contour,(pos[0] + 3, pos[1] + 3))
            self.surface_affichage.blit(contour,(pos[0] + 3, pos[1] - 3))
            self.surface_affichage.blit(texte,rect_texte)

    def evt_lettre(self):
        '''
        Effectue la lecture de la lettre du père de Kuuki (le joueur). 
        Le texte affiché est déterminé par la présente valeur de index_texte qui permet de prendre une certaine surface dans la liste de textes.
        Si l'indice dépasse la longueur de cette liste, l'événement se termine (réinitialisation des booléens)
        '''
        self.joueur.direction.x = 0 # Remet le joueur au statut "sur_place"
        self.joueur.jeu_enc = False # Le joueur ne peut plus bouger
        

        self.surface_affichage.blit(self.assombrissement,(0,0))
        self.surface_affichage.blit(self.lettre_grand,(largeur_ecran // 2 - 100, 150))

        self.affichage_texte(self.textes_lettre[self.index_texte],self.boite_dialogue,self.rect_boite_dialogue)

        self.clic()
        
        if self.index_texte >= len(self.textes_lettre):
            self.index_texte = 0
            self.passage_texte = False 
            self.joueur.jeu_enc = True


    def clic(self):
        '''
        Fonction semblable dans le principe au clic() des boutons. Seulement il faut que le clic ne s'active qu'une seule fois par clic de souris,
        peu importe le temps de clic, il y a donc quelques modifications à faire au niveau des booléens et des instructions conditionnelles.
        '''

        clic = False

        if pg.mouse.get_pressed()[0] and not self.action: 
            # Activation de booléens si le bouton gauche de la souris est cliqué.
            clic = True
            self.action = True
        if clic and self.action:
            # Incrémente l'indice de texte de 1 pour passer à la chaîne de caractère suivante dans la liste de textes (autrement dit aux lignes suivantes dans un dialogue)
            self.index_texte += 1
            clic = False

        elif self.action and not pg.mouse.get_pressed()[0]: 
            # Tant que le bouton est pressé, l'action sera toujours en cours.
            self.action = False

    def affichage_statistiques(self,nb_KO,nb_collectables,tps_total):
        '''
        Affiche les statistiques de jeu sur l'écran de fin : nombre de KO et de cristaux récupérés, temps de complétion en h:min:s 
        et un message de félicitations si l'on a récupéré tous les cristaux.
        Anime un cristal à côté des stats de cristaux.
        '''
        liste_stats = [f"Nombre total de KO : {nb_KO}",f"Temps total de complétion : {tps_total[0]}:{tps_total[1]}:{tps_total[2]}.{tps_total[3]}",\
            f"Nombre de cristaux récupérés : {nb_collectables}"]
        if nb_collectables == 6 and len(liste_stats) < 5: liste_stats.append("Bravo ! Vous avez récupéré tous les cristaux !")
        stats = self.textes(liste_stats,300,100)
        self.affichage_texte(stats,0,0,False)

        self.groupe_potion.update()
        self.groupe_potion.draw(self.surface_affichage)

    def mesure_temps(self):
        '''
        Mesure le temps en utilisant la fonction get_ticks() du module time de Pygame. Cette fonction renvoie le nombre de millisecondes depuis le pg.init().
        À partir de ça, dès que le joueur fait autre chose que jouer, le compte s'arrête et la valeur de temps_passe est remise à zéro après avoir été ajoutée au temps total en secondes.
        temps_passe est réinitialisé en actualisant la valeur de temps_depart et en lui retranchant cette dernière.
        Lorsque le niveau est terminé, le temps de complétion est converti en min:s.ms puis affiché sur l'écran de transition (plus tard).
        '''
        if self.joueur.KO or self.menu or not self.joueur.jeu_enc or self.changement_niveau:
            if self.temps_passe > 0: 
                self.temps_completion += self.temps_passe * 10**-3
                round(self.temps_completion,3)
                self.temps_en_sec = self.temps_completion
                if self.changement_niveau:
                    self.temps_completion = [int(self.temps_completion // 60),int(self.temps_completion) % 60,int(self.temps_completion % 1 * 1000)]
                    if self.temps_completion[1] < 10: self.temps_completion[1] = "0" + str(self.temps_completion[1])
            self.temps_depart = pg.time.get_ticks()
            self.temps_passe = 0
        else:
            self.temps_passe = pg.time.get_ticks() - self.temps_depart


    


    def disposition_niveau(self):
        '''
        Appelle toutes les fonctions d'affichage de texture des différents groupes 
        et utilise les méthodes de certains groupes particuliers pour effectuer certaines actions.
        S'occupe également de toutes les autres choses nécessitant une actualisation régulière.
        '''

        # Méthodes concernant l'affichage de la feuille de saut   
        # Création d'un sprite de feuille dont le rectangle suit celui du joueur lorsque ce dernier effectue un double saut.
        # La feuille ne disparaît pas tant que le joueur n'a pas touché le sol.
        if self.joueur.feuille and not self.joueur.a_saute and not self.joueur.en_chute: 
            self.feuille = FeuilleSaut(0,self.groupe_sprites,self.joueur)
            self.joueur.feuille = False # Booléen qui s'active au moment du double saut et permet ainsi la création du sprite (voir fichier joueur). On le désactive à la fin

        # Affichage de la feuille de saut en tant qu'objet à collecter. 
        # Elle est enlevée de tous les groupes et active le double saut (ds) du joueur après que ce dernier l'a touchée. Plus de précisions dans le fichier tuiles.
        if self.info_niveau == niveau_3 and not self.joueur.ds_actif:
            if self.groupe_feuille_saut.has(self.feuille_saut): self.joueur.ds_actif = self.groupe_feuille_saut.sprite.obtention(self.joueur)
            if self.joueur.ds_actif: self.joueur.en_chute = False # Autrement le double saut s'activait bizarrement et faisait sortir le joueur de l'écran si l'on gardait la barre espace enfoncée


        # Mouvement de la décoration, voir le fichier decoration
        self.lucioles.mouvement()
        if not self.deuxieme_partie: self.feuilles.mouvement()

        # Mise à jour des sprites et affichage des sprites selon la position du rectangle de la caméra
        # Mise à jour des sprites du groupe végétation après celle du groupe des sprites restants pour que l'herbe et les lianes se superposent à tous les autres sprites
        self.groupe_sprites.update()
        self.groupe_sprites.affichage_sprites(self.joueur,self.largeur_niveau,self.hauteur_niveau)

        self.groupe_vegetation.update()
        self.groupe_vegetation.affichage_sprites(self.joueur,self.largeur_niveau,self.hauteur_niveau)

        # Mise à jour de l'événement de lecture de lettre dans le niveau 1.
        # Je le mets ici pour afficher l'image de la lettre au-dessus de tous les sprites lors de la lecture (instruction elif)
        if self.info_niveau == niveau_1:
            if self.groupe_lettre.has(self.lettre): # Instruction effectuée tant que le groupe lettre possède un sprite du type Lettre
                if self.groupe_lettre.sprite.lecture_texte(self.joueur): # Si le joueur touche la lettre, on l'enlève de tous les groupes et on passe ainsi à l'instruction elif
                    self.groupe_lettre.sprite.kill() # Supprime le sprite de tous les groupes où il est présent
                    self.passage_texte = True # Booléen nécessaire pour empêcher le jeu de se mettre en pause
            elif self.passage_texte: self.evt_lettre()

        # Affichage du rectangle d'assombrissement sur le niveau de fin (il se superpose à toutes les autres textures, sauf lorsque le jeu est en pause ou dans le menu principal)
        if self.info_niveau == fin and self.joueur.jeu_enc: self.surface_affichage.blit(self.assombr_fin,(0,0))

        # Appel de la méthode du sprite Portail qui permet d'activer le fondu et de changer l'indice de niveau
        # Je ne peux pas la mettre plus bas : il faut la placer avant l'appel des trois méthodes de Niveau
        if not self.changement_niveau and self.info_niveau != fin: self.niveau_max,self.changement_niveau = self.groupe_arrivee.sprite.arrivee(self.joueur)

        # Appel des trois méthodes qui doivent être utilisées à chaque frame
        self.menus()
        self.fondu()
        self.mesure_temps()

        # Appel de méthodes en fonction du dictionnaire de niveau utilisé
        if self.info_niveau != fin:

            if not self.compteur_collectable:
                # Si le cristal n'est pas déjà récupéré, le compteur est mis à True si le joueur touche le cristal plus tard.
                if self.groupe_collectable.sprite.obtention(self.joueur): self.compteur_collectable = True

            if self.groupe_arrivee.sprite.arrive and not self.changement_niveau: 
                # Après la première partie d'animation de fondu qui suit l'arrivée en fin de niveau sont affichées les statistiques relatives au niveau.
                # Dans l'ordre : KO puis temps.
                liste_stats = [f"Nombre de KO : {self.compteur_mort}",f"Temps de complétion :  {self.temps_completion[0]}:{self.temps_completion[1]}.{self.temps_completion[2]}"]
                KO = self.textes(liste_stats,350,100)
                self.affichage_texte(KO,0,0,False)

            # Méthode des sprites Checkpoint qui redéfinit la position de départ au passage d'un checkpoint.
            for sprite in self.groupe_check.sprites(): self.pos_dep = sprite.point_de_passage(self.pos_dep,self.joueur)


        else: 
            # Si le dictionnaire est celui du dernier niveau.
            if not self.changement_niveau and self.groupe_potion.has(self.potion): 
                # On se sert ici du booléen pour déclencher l'animation de fondu menant à l'écran de fin lorsque le joueur touche la potion
                self.changement_niveau = self.groupe_potion.sprite.obtention(self.joueur)
            elif self.changement_niveau:
                if self.groupe_potion.has(self.potion): 
                    # Si on passe à l'écran de fin et que le groupe potion a encore son sprite, on supprime le sprite de tous les groupes
                    # puis  on crée un sprite de cristal, affiché à côté des statistiques relatives à la collection des cristaux.
                    self.groupe_potion.sprite.kill()
                    collectable = TuileAnimée(0,(255,485),"../Visuels/collectable/sur_place")
                    self.groupe_potion.add(collectable)
                    
                
class GroupeCamera(pg.sprite.Group):
    '''
    Classe qui crée un rectangle plus petit que l'écran d'affichage et qui permet de déplacer les textures sur l'écran si le joueur en dépasse les limites.
    Les dimensions du rectangle sont déterminées à partir de la taille de l'écran 
    et de variables qui correspondent à l'écart entre les côtés correspondants des rectangles de l'écran et de la caméra.
    '''
    def __init__(self):
        super().__init__()
        self.surf_affich = pg.display.get_surface() # Permet d'extraire la surface correspondant à la surface d'affichage
        self.decalage = pg.math.Vector2() # Sorte de tuple contenant deux composantes de vecteur. Permet d'effectuer des déplacements de rectangles.

        # Rectangle limite de la caméra
        gauche_cam = limites_camera["gauche"] # Création de ces deux variables pour pouvoir ensuite placer le rectangle de la caméra
        haut_cam = limites_camera["haut"]

        # Dimensions du rectangle. get_size() renvoie un tuple correspondant aux dimensions de l'image en question
        largeur_cam = self.surf_affich.get_size()[0] - (limites_camera["droite"] + gauche_cam) 
        hauteur_cam = self.surf_affich.get_size()[1] - (limites_camera["bas"] + haut_cam)

        self.rect_camera = pg.Rect(gauche_cam,haut_cam,largeur_cam,hauteur_cam)

    def affichage_sprites(self,joueur,largeur_niveau,hauteur_niveau): 

        # Obtention de la position de la caméra et déplacement du rectangle de la caméra
        # Les deuxièmes conditions fixent les limites de déplacement de la caméra. Au-delà de ces valeurs, la caméra ne peut plus se déplacer (ce sont les zones de vide que l'on ne veut pas voir)
        if joueur.rect_collisions.left < self.rect_camera.left and joueur.rect_collisions.left > limites_camera["gauche"]: self.rect_camera.left = joueur.rect_collisions.left
        if joueur.rect_collisions.right > self.rect_camera.right and joueur.rect_collisions.right < largeur_niveau - limites_camera["droite"]: self.rect_camera.right = joueur.rect_collisions.right
        if joueur.rect_collisions.top < self.rect_camera.top and joueur.rect_collisions.top > limites_camera["haut"]: self.rect_camera.top = joueur.rect_collisions.top
        if joueur.rect_collisions.bottom > self.rect_camera.bottom and joueur.rect_collisions.bottom < hauteur_niveau - limites_camera["bas"]: self.rect_camera.bottom = joueur.rect_collisions.bottom

        # Je mets les collisions de limites de niveau ici parce que je dispose déjà de toutes les variables nécessaires
        # Elles empêchent le joueur de tomber s'il décide d'aller trop à gauche ou à droite dans le niveau
        if joueur.rect_collisions.x <= 20: joueur.rect_collisions.x = 20
        elif joueur.rect_collisions.x >= largeur_niveau - 40: joueur.rect_collisions.x = largeur_niveau - 40 


        # Décalage de la caméra
        self.decalage = pg.math.Vector2(self.rect_camera.left - limites_camera['gauche'],self.rect_camera.top - limites_camera['haut'])

        
        for sprite in self.sprites():
            # Calcule la nouvelle position des sprites sur l'écran
            # Le décalage est enlevé à leur position pour faire avancer la caméra en même temps que le joueur
            # Lorsque le joueur dépasse le rectangle de la caméra à gauche par exemple, les sprites se déplacent vers la droite 
            # (soustraction de la première composante du vecteur décalage négative)
            pos_decalee = sprite.rect.topleft - self.decalage

            # Affichage des textures avec la nouvelle position. Je me suis dit que si je n'affiche que les texures proches des limites de l'écran mais à l'extérieur,
            # Je sauvegardais un peu de mémoire. Je n'ai pas remarqué de différence fondamentale, mais ça peut toujours servir.
            if sprite.rect.right < self.rect_camera.right + 2 * limites_camera["droite"] and sprite.rect.left > self.rect_camera.left - 2 * limites_camera["gauche"]:
                if sprite.rect.top > self.rect_camera.top - 2 * limites_camera["haut"] and sprite.rect.bottom < self.rect_camera.bottom + 2 * limites_camera["bas"]:
                    self.surf_affich.blit(sprite.image,pos_decalee) 
        

            
        

        